# Generated by Django 3.2.4 on 2021-06-20 22:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mfa', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='mfakey',
            name='last_code',
            field=models.CharField(blank=True, max_length=32),
        ),
    ]
