# Generated by Django 3.2.4 on 2021-07-31 06:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mfa', '0002_mfakey_last_code'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mfakey',
            name='method',
            field=models.CharField(choices=[('FIDO2', 'FIDO2'), ('TOTP', 'TOTP'), ('recovery', 'recovery')], max_length=8),
        ),
    ]
