{% load mfa %}

<p>Scan the code with your TOTP app and enter a valid code to finish registration.</p>

{{ mfa_data.url|qrcode }}
{{ mfa_data.secret }}

<form method="POST">
    {% csrf_token %}
    {% for error in form.non_field_errors %}
        <p>{{ error }}</p>
    {% endfor %}
    <label>
        {{ form.name.label }}
        {{ form.name }}
    </label>
    <label>
        {{ form.code.label }}
        {{ form.code }}
    </label>
    <button>Create</button>
</form>
