<h1> {% if object_list %}Login keys configured{% else %}No login keys configured{% endif %}</h1>
<p>Two-factor authentication adds an additional layer of security to your account by requiring more than just a password to log in.</p>

{% if object_list|length == 1 %}
    <p>
        <strong>WARNING</strong>:
        You have registered only a single key. If you lose access to that key you won't be able log into your account again.
        <a href="{% url 'mfa:create' 'recovery' %}">Create recovery code</a>
    </p>
{% endif %}

<ul>
    {% for key in object_list %}
        <li>
            {{ key.name }} ({{ key.method }})
            <a href="{% url 'mfa:delete' key.id %}">Delete</a>
        </li>
    {% endfor %}
</ul>

<a href="{% url 'mfa:create' 'TOTP' %}">Create TOTP key</a>
<a href="{% url 'mfa:create' 'FIDO2' %}">Create FIDO2 key</a>
<a href="{% url 'mfa:create' 'recovery' %}">Create recovery code</a>
